//************************** FIGURE #2 ********************************
//**                                                                 **
//**   SERIAL PROGRAMMING ROUTINE FOR THE PIC16C84 MICROCONTROLLER   **
//**                                                                 **
//**                   Analog Design Specialists                     **
//**                                                                 **
//*********************************************************************

//FUNCTION PROTOTYPE: int ser_pic16c84(int cmd, int data)

// cmd:	LOAD_CONFIG   -> part configuration bits
//	LOAD_DATA     -> program data, write
//	READ_DATA     -> program data, read
//      INC_ADDR      -> increment to the next address (routine does not auto increment)
//	BEGIN_PROG    -> program a previously loaded program code or data
//	LOAD_DATA_DM  -> load EEPROM data regesters (BEGIN_PROG must folow)
//	READ_DATA_DM  -> read EEPROM data
//
// data: 1) 14 bits of program data or
//       2)  8 bits of EEPROM data (least significant 8 bits of int)

// Additional programmer commands (not part of PIC16C84 programming codes)
//
// cmd:	RESET         -> provides 1 ms reset pulse to target system
//      PROGRAM_MODE  -> initializes PIC16C84 for programming
//      RUN           -> disconnects programmer from target system
//
// function returns: 1) 14 or 8 bits read back data for read commands
//		     2) zero                        for write data commands
//		     3) PIC_PROG_EROR = -1 for programming function errors
//		     4) PROGMR_ERROR  = -2 for programmer function errors


#include <bios.h>

#define LOAD_CONFIG    0
#define LOAD_DATA      2
#define READ_DATA      4
#define INC_ADDR       6
#define BEGIN_PROG     8
#define PARALLEL_MODE 10  // not used
#define LOAD_DATA_DM   3
#define READ_DATA_DM   5
#define MAX_PIC_CMD   63  // division between pic and programmer commands

#define RESET         64  // external reset command, not neded for programming
#define PROGRAM_MODE  65  // initialize program mode
#define RUN           66  // electrically disconnect programmer

#define PIC_PROG_EROR -1
#define PROGMR_ERROR  -2

#define PTR            0  // use device #0

// parallel port bits
//	d0: data output to part to be programmed
//	d1: programming clock
//	d2: data dirrection, 0= enabel tri state buf -> send data to part
//	d3: Vpp control 1= turn on Vpp
//	d4: ~MCLR =0, 1 = reset device with MCLR line
//	d5: clock line tri state control, 0 = enable clock line

int ser_pic16c84(int cmd, int data)     // custom interface for pic 16c84
  {
  int i, s_cmd;

  if(cmd <=MAX_PIC_CMD)                 // all programming modes
    {
    biosprint(0,8,PTR); 		// set bits 001000, output mode, clock & data low
    s_cmd = cmd;                        // retain command "cmd"
    for (i=0;i<6;i++)              	// output 6 bits of command
      {
      biosprint(0,(s_cmd&0x1) +2+8,PTR);// set bits 001010, clock hi
      biosprint(0,(s_cmd&0x1)   +8,PTR);// set bits 001000, clock low
      s_cmd >>=1;
      }

    if((cmd ==INC_ADDR)||(cmd ==PARALLEL_MODE))// command only, no data cycle
      return 0;

    else if(cmd ==BEGIN_PROG)           // program command only, no data cycle
      {
      delay(10);                        // 10 ms PIC programming time
      return 0;
      }

    else if((cmd ==LOAD_DATA)||(cmd ==LOAD_DATA_DM)||(cmd ==LOAD_CONFIG)) // output 14 bits of data
      {
      for (i=200;i;i--) ; 		// delay between command & data
      biosprint(0,2+8,PTR);    		// set bits 001010, clock hi; leading bit
      biosprint(0,  8,PTR);    		// set bits 001000, clock low

      for (i=0;i<14;i++)        	// 14 data bits, lsb first
	{
	biosprint(0,(data&0x1) +2+8,PTR);// set bits 001010,  clock hi
	biosprint(0,(data&0x1)   +8,PTR);// set bits 001000,  clock low
	data >>=1;
	}
      biosprint(0,2+8,PTR);    		// set bits 001010, clock hi; trailing bit

   // *****************  Analog Design Specialists  ******************


      biosprint(0,  8,PTR);    		// set bits 001000, clock low

      return 0;
      }

    else if((cmd ==READ_DATA)||(cmd ==READ_DATA_DM)) //read 14 bits from part, lsb first
      {
      biosprint(0,  4+8,PTR);    	// set bits 001100, clock low, tri state data buffer
      for (i=200;i;i--) ; 		// delay between command & data
      biosprint(0,2+4+8,PTR);    	// set bits 001110, clock hi, leading bit
      biosprint(0,  4+8,PTR);    	// set bits 001100, clock low

      data =0;
      for (i=0;i<14;i++)             	// input 14 bits of data, lsb first
	{
	data >>=1;                      // shift data for next input bit
	biosprint(0,2+4+8,PTR); 	// set bits 001110,  clock hi
	biosprint(0,  4+8,PTR); 	// set bits 001100,  clock low
	if(!(biosprint(2,0,0)&0x40)) data += 0x2000;//use printer acknowledge line for input, data lsb first
	}
      biosprint(0,2+4+8,PTR);    	// set bits 001110, clock hi, trailing bit
      biosprint(0,  4+8,PTR);    	// set bits 001100, clock low
      return data;
      }

    else return PIC_PROG_EROR;          // programmer error

    }
  else if(cmd == RESET)                 // reset device
    {
    biosprint(0,32+16+4,PTR); 		// set bits 110100, MCLR = low (reset PIC16C84), programmer not conected
    delay(1);                           // 1ms delay
    biosprint(0,32   +4,PTR); 		// set bits 100100, MCLR = high
    return 0;
    }

  else if(cmd ==PROGRAM_MODE)           // enter program mode
    {
    biosprint(0,32+16+4,PTR);    	// set bits 110100, Vpp off, MCLR = low (reset PIC16C84)
    delay(10);              		// 10 ms, allow programming voltage to stabelize

    biosprint(0,8,PTR);     		// set bits 001000, Vpp on , MCLR = 13.5 volts, clock & data connected
    delay(10);              		// 10 ms, allow programming voltage to stablize

    return 0;
    }

  else if(cmd ==RUN)          		// disconects programmer from device
    {
    biosprint(0,32+4,PTR);  		// set bits 100100
    return 0;
    }
  else return PROGMR_ERROR;             // command error
  }

